package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 游记中的图片操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "游记中的图片管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourTravelNotesImg")
public class TourTravelNotesImgController {

    @Autowired
    private TourTravelNotesImgService tourTravelNotesImgService;

    /**
     * 新增游记中的图片数据。
     *
     * @param tourTravelNotesImgDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourTravelNotesImgDto.id"})
    @SaCheckPermission("tourTravelNotesImg.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourTravelNotesImgDto tourTravelNotesImgDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourTravelNotesImgDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourTravelNotesImg tourTravelNotesImg = MyModelUtil.copyTo(tourTravelNotesImgDto, TourTravelNotesImg.class);
        tourTravelNotesImg = tourTravelNotesImgService.saveNew(tourTravelNotesImg);
        return ResponseResult.success(tourTravelNotesImg.getId());
    }

    /**
     * 更新游记中的图片数据。
     *
     * @param tourTravelNotesImgDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesImg.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourTravelNotesImgDto tourTravelNotesImgDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourTravelNotesImgDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourTravelNotesImg tourTravelNotesImg = MyModelUtil.copyTo(tourTravelNotesImgDto, TourTravelNotesImg.class);
        TourTravelNotesImg originalTourTravelNotesImg = tourTravelNotesImgService.getById(tourTravelNotesImg.getId());
        if (originalTourTravelNotesImg == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourTravelNotesImgService.update(tourTravelNotesImg, originalTourTravelNotesImg)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除游记中的图片数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesImg.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除游记中的图片数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesImg.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的游记中的图片列表。
     *
     * @param tourTravelNotesImgDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourTravelNotesImg.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourTravelNotesImgVo>> list(
            @MyRequestBody TourTravelNotesImgDto tourTravelNotesImgDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourTravelNotesImg tourTravelNotesImgFilter = MyModelUtil.copyTo(tourTravelNotesImgDtoFilter, TourTravelNotesImg.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourTravelNotesImg.class);
        List<TourTravelNotesImg> tourTravelNotesImgList =
                tourTravelNotesImgService.getTourTravelNotesImgListWithRelation(tourTravelNotesImgFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourTravelNotesImgList, TourTravelNotesImgVo.class));
    }

    /**
     * 查看指定游记中的图片对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourTravelNotesImg.view")
    @GetMapping("/view")
    public ResponseResult<TourTravelNotesImgVo> view(@RequestParam Long id) {
        TourTravelNotesImg tourTravelNotesImg = tourTravelNotesImgService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourTravelNotesImg == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourTravelNotesImgVo tourTravelNotesImgVo = MyModelUtil.copyTo(tourTravelNotesImg, TourTravelNotesImgVo.class);
        return ResponseResult.success(tourTravelNotesImgVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourTravelNotesImg originalTourTravelNotesImg = tourTravelNotesImgService.getById(id);
        if (originalTourTravelNotesImg == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourTravelNotesImgService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
