package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 游记与话题关联操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "游记与话题关联管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourTravelNotesTopicRelation")
public class TourTravelNotesTopicRelationController {

    @Autowired
    private TourTravelNotesTopicRelationService tourTravelNotesTopicRelationService;

    /**
     * 新增游记与话题关联数据。
     *
     * @param tourTravelNotesTopicRelationDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourTravelNotesTopicRelationDto.id"})
    @SaCheckPermission("tourTravelNotesTopicRelation.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourTravelNotesTopicRelationDto tourTravelNotesTopicRelationDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourTravelNotesTopicRelationDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourTravelNotesTopicRelation tourTravelNotesTopicRelation = MyModelUtil.copyTo(tourTravelNotesTopicRelationDto, TourTravelNotesTopicRelation.class);
        tourTravelNotesTopicRelation = tourTravelNotesTopicRelationService.saveNew(tourTravelNotesTopicRelation);
        return ResponseResult.success(tourTravelNotesTopicRelation.getId());
    }

    /**
     * 更新游记与话题关联数据。
     *
     * @param tourTravelNotesTopicRelationDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesTopicRelation.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourTravelNotesTopicRelationDto tourTravelNotesTopicRelationDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourTravelNotesTopicRelationDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourTravelNotesTopicRelation tourTravelNotesTopicRelation = MyModelUtil.copyTo(tourTravelNotesTopicRelationDto, TourTravelNotesTopicRelation.class);
        TourTravelNotesTopicRelation originalTourTravelNotesTopicRelation = tourTravelNotesTopicRelationService.getById(tourTravelNotesTopicRelation.getId());
        if (originalTourTravelNotesTopicRelation == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourTravelNotesTopicRelationService.update(tourTravelNotesTopicRelation, originalTourTravelNotesTopicRelation)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除游记与话题关联数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesTopicRelation.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除游记与话题关联数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesTopicRelation.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的游记与话题关联列表。
     *
     * @param tourTravelNotesTopicRelationDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourTravelNotesTopicRelation.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourTravelNotesTopicRelationVo>> list(
            @MyRequestBody TourTravelNotesTopicRelationDto tourTravelNotesTopicRelationDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourTravelNotesTopicRelation tourTravelNotesTopicRelationFilter = MyModelUtil.copyTo(tourTravelNotesTopicRelationDtoFilter, TourTravelNotesTopicRelation.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourTravelNotesTopicRelation.class);
        List<TourTravelNotesTopicRelation> tourTravelNotesTopicRelationList =
                tourTravelNotesTopicRelationService.getTourTravelNotesTopicRelationListWithRelation(tourTravelNotesTopicRelationFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourTravelNotesTopicRelationList, TourTravelNotesTopicRelationVo.class));
    }

    /**
     * 查看指定游记与话题关联对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourTravelNotesTopicRelation.view")
    @GetMapping("/view")
    public ResponseResult<TourTravelNotesTopicRelationVo> view(@RequestParam Long id) {
        TourTravelNotesTopicRelation tourTravelNotesTopicRelation = tourTravelNotesTopicRelationService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourTravelNotesTopicRelation == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourTravelNotesTopicRelationVo tourTravelNotesTopicRelationVo = MyModelUtil.copyTo(tourTravelNotesTopicRelation, TourTravelNotesTopicRelationVo.class);
        return ResponseResult.success(tourTravelNotesTopicRelationVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourTravelNotesTopicRelation originalTourTravelNotesTopicRelation = tourTravelNotesTopicRelationService.getById(id);
        if (originalTourTravelNotesTopicRelation == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourTravelNotesTopicRelationService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
